#ifndef COMMON_H
#define COMMON_H

/**
   Macro for 0-based indexing access to a matrix. We will use
   column-major storage because CUBLAS use this kind of storage as explained in
   <a href="http://developer.download.nvidia.com/compute/cuda/4_0/toolkit/docs/CUBLAS_Library.pdf">CUDA TOOLKIT 4.0 CUBLAS Library</a>, page 4.
*/
#define IDX2C(i,j,rows) (((j)*(rows))+(i))

#define MIB 1048576.0

#include <stdio.h>  // FILE
#include <stdlib.h> // struct timeval


struct options_t {
  int  device;      // Id of CUDA device to be used
  int  device2;     // Id of second CUDA device to be used
  int  device_info; // Show device information?
  int  m, n, k;     // Rows and cols of matrices A, B and C in gemm and gemv
  int  rows, cols;  // Rows and cols in tests that use only one matrix
  int  padding;     // Use padding?
  int  rep;         // Number of repetitions of each test
  int  use_simple;  // Whether to use simple or double precision
  int  help;        // Show the usage help?
  FILE *output;     // Pointer to output FILE (stdout or a real file)
  int  verbosity;   // Verbosity level
};

void getopts(int argc, char **argv, struct options_t *option);

void error(char *msg, ...);
void no_exit_error(char *msg, ...);
void warning(char *msg, ...);

//@todelete: unless they are required for something that has nothing to do with computing the mean
//int compare_float(const void *a, const void *b);
//int compare_double(const void *a, const void *b);

double mean(double *vector, int size);

double timeval_diff(struct timeval t1, struct timeval t0);

#endif
