/*==============================================================================
 * gb_common_cuda.cu
 *------------------------------------------------------------------------------
 *
 * Copyright 2009-2012 Universitat Jaume I
 *
 * This file is part of GPUBenchmark.
 *
 * GPUBenchmark is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GPUBenchmark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GPUBenchmark.  If not, see <http://www.gnu.org/licenses/>.
 *==============================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <cuda.h>
#include <cuda_runtime_api.h>
#include "gb_common_cuda.h"
extern "C" {
#include "gb_common.h"
}

void showProperties(FILE *out, int device)
{
  int v_driver, v_runtime;
  cudaDeviceProp properties;

  if(cudaGetDeviceProperties(&properties, device) != cudaSuccess)
    error("Attemp to get properties of device %d failed.\n", device);

  cudaDriverGetVersion(&v_driver);
  cudaRuntimeGetVersion(&v_runtime);

  fprintf(out, "%s\n", properties.name);
  fprintf(out, " CUDA driver version             : %d\n", v_driver);
  fprintf(out, " CUDA Runtime version            : %d\n", v_runtime);
  fprintf(out, " Multiprocessors                 : %d\n", properties.multiProcessorCount);
  fprintf(out, " Global memory (total)           : %u bytes\n", properties.totalGlobalMem);
  fprintf(out, " Constant memory (total)         : %u bytes\n", properties.totalConstMem); 
  fprintf(out, " Shared memory per block (total) : %u bytes\n", properties.sharedMemPerBlock);
  fprintf(out, " Available registers per block   : %d\n", properties.regsPerBlock);  
  fprintf(out, " Threads per block               : %d\n", properties.maxThreadsPerBlock);
  fprintf(out, " Max. dimension of a block       : %d x %d x %d\n", properties.maxThreadsDim[0], properties.maxThreadsDim[1], properties.maxThreadsDim[2]);
  fprintf(out, " Max. dimension of a grid        : %d x %d x %d\n", properties.maxGridSize[0], properties.maxGridSize[1], properties.maxGridSize[2]);
  fprintf(out, " Clock rate                      : %.2f GHz\n", properties.clockRate * 1e-6f);

  exit(0);
}
