#!/bin/bash

#===============================================================================
# gb_gputogpu_run.sh
#-------------------------------------------------------------------------------
#
# Copyright 2009-2012 Universitat Jaume I
#
# This file is part of GPUBenchmark.
#
# GPUBenchmark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# GPUBenchmark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GPUBenchmark.  If not, see <http://www.gnu.org/licenses/>.
#===============================================================================


#===============================================================================
# Tests parameters
#-------------------------------------------------------------------------------
#  - rows and cols must have the same number of elements
gputogpu_rows=(128 256 512 1024 2048 4096 8192 10240)
gputogpu_cols=(128 256 512 1024 2048 4096 8192 10240)

#===============================================================================


#===============================================================================
# Check that we are run from gb_run.sh
#-------------------------------------------------------------------------------
if [ -z "${GB_DIR}" ]; then
    echo "Error: this file must be called from 'gb_run.sh'"
fi
#===============================================================================


#===============================================================================
# gputogpu test
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Checks and initialization of variables
#-------------------------------------------------------------------------------
if [ ${#gputogpu_rows[@]} -ne ${#gputogpu_cols[@]} ]; then
   echo "Error: gputogpu parameters must have the same number of rows than columns"
   exit -1
fi
ELEM=$[ ${#gputogpu_rows[@]} - ${ONE} ]
TEST=gb_gputogpu
TEST_EXE="${TEST}/${TEST}"
TEST_OUTPUT_DIR="${GB_OUTPUT_DIR}/${TEST}"
[ -d "${TEST_OUTPUT_DIR}" ] || mkdir -p "${TEST_OUTPUT_DIR}"
TEST_OUTPUT_FILE="${TEST_OUTPUT_DIR}/${TEST}.out"
[ -e "${TEST_OUTPUT_FILE}" ] && rm -f "${TEST_OUTPUT_FILE}"

#-------------------------------------------------------------------------------
# Check that the test can be run. Otherwise, remove it from the report and exit
#-------------------------------------------------------------------------------
${TEST_EXE} --rows=10 --cols=10 >/dev/null 2>&1
if [ $? -ne 0 ]; then
    sed --in-place -e 's/\\input{gb_gputogpu.tex}//' ${MACROS_FILE}
    exit
fi

#-------------------------------------------------------------------------------
# Actual running of test
#-------------------------------------------------------------------------------
echo -n "Running transfer speed test between two gpus: "
progress_bar ${ELEM}
for i in $(seq 0 ${ELEM}); do
    ${TEST_EXE} \
	--rows=${gputogpu_rows[$i]} \
	--cols=${gputogpu_cols[$i]} \
	>> ${TEST_OUTPUT_FILE}
    progress_bar_step $?
done
echo

#-------------------------------------------------------------------------------
# Create gnuplot data files from results
#-------------------------------------------------------------------------------
DATA_FILE="${TEST_OUTPUT_FILE%.out}.data"
grep -E -v '^#|^$' ${TEST_OUTPUT_FILE} > ${DATA_FILE}

#-------------------------------------------------------------------------------
# Create macros for latex document tables with rows, cols, size, and speeds
#-------------------------------------------------------------------------------
gputogpu_table="define(GPUTOGPUTABLE,"
for i in $(seq 0 ${ELEM}); do
    j=$[ $i + 1 ]
    gputogpu_table="$gputogpu_table "\
"$(awk 'NR=='$j' {print $1, "&", $2, "&", $3, "&", $6, "&", $7 }' ${DATA_FILE}) \\\\"
done
echo "$gputogpu_table)" >> ${MACROS_FILE}

#-------------------------------------------------------------------------------
# Create gnuplot file
#-------------------------------------------------------------------------------
GP_FILE="${TEST_OUTPUT_FILE%.out}.gp"
# Files called from the gnuplot file must be relative to GB_OUPUT_DIR, not to GB_DIR
REL_DATA_FILE="${DATA_FILE/${GB_OUTPUT_DIR}\/}"
REL_EPS_FILE="${TEST}/${TEST}.eps"
cat > ${GP_FILE}  <<EOF
set xlabel "Size of matrix (MiB)"
set ylabel "Transfer speed (MiB/s)"
set key bottom right nobox
set grid
set logscale x
set terminal postscript enhanced color
set output "${REL_EPS_FILE}"
plot "${REL_DATA_FILE}" u 3:6 t 'GPU1->GPU2' w linespoints lw 2, "" u 3:7 t 'GPU1->MM->GPU2' w linespoints lw 2
EOF
#===============================================================================
