#!/bin/bash

#===============================================================================
# gb_make_pdf.sh
#-------------------------------------------------------------------------------
#
# Copyright 2009-2012 Universitat Jaume I
#
# This file is part of GPUBenchmark.
#
# GPUBenchmark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# GPUBenchmark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GPUBenchmark.  If not, see <http://www.gnu.org/licenses/>.
#===============================================================================


#===============================================================================
# Initialization and checks
#-------------------------------------------------------------------------------
GB_DIR=$(dirname $0)

if [ ! -z "$(echo ${GB_DIR} | grep -E 'templates$')" -o \
     ! -z "$(echo ${PWD} | grep -E 'templates$')" ]; then
    echo "Error: this program must be run from a gpubenchmark results directory."
    exit -1
fi

# Go into GB_DIR
cd ${GB_DIR}
GB_OUTPUT_DIR="./"

REQUIRED_APPS="gnuplot ps2pdf m4 pdflatex"

for app in ${REQUIRED_APPS}; do
    which ${app}  > /dev/null 2>&1
    if [ $? -ne 0 ]; then
	cat <<EOF

The required application '${app}' has not been found. Please, install
it, or if it is already installed, add its path to PATH.

If you are not allowed to install this application on this computer,
then you can copy this directory to another computer and run this
script, 'gb_make_pdf.sh', on it.

EOF
	exit -1
    fi
done

TEMPLATE="GPUBenchmark_results_tpl.tex"

# As this script can be run in a different machine than the one were
# the tests were done, we can not use the variable HOSTNAME to
# determine the name of the output tex file. Instead, we use whatever
# it is in the hostname.txt file on this directory (gb_run.sh should
# have writen the correct hostname on it).
TEX_FILE="GPUBenchmark_$(cat ./hostname.txt)_results.tex"
#===============================================================================


#===============================================================================
# Process the gnuplots
#-------------------------------------------------------------------------------
echo "Generating the figures..."

for gp in $(find ${GB_OUTPUT_DIR} -name "*.gp"); do
    gnuplot "${gp}"
done
for eps in $(find ${GB_OUTPUT_DIR} -name "*.eps"); do
    ps2pdf "${eps}" "${eps%.eps}.pdf"
    rm -f "${eps}"
    mv "${eps%.eps}.pdf" ${GB_OUTPUT_DIR}
done
#===============================================================================


#===============================================================================
# Process the LaTeX file
#-------------------------------------------------------------------------------
echo "Generating the LaTeX file..."
MACROS_FILE="macros.txt"

for FILE in "./templates/${TEMPLATE}" "${MACROS_FILE}"; do
    if [ ! -e "${FILE}" ]; then
	echo "Error: '${FILE}' not found. Do you know where it is?"
	exit -1
    fi
done

# m4 processing
for FILE in ./templates/*_tpl.tex; do
    OFILE=$(basename ${FILE%_tpl.tex}.tex)
    m4 "${MACROS_FILE}" "${FILE}" > "./${OFILE}"
done
mv "${TEMPLATE%_tpl.tex}.tex" "${TEX_FILE}"
#===============================================================================


#===============================================================================
# Pdflatex
#-------------------------------------------------------------------------------
echo "Executing pdflatex on the generated latex file..."
# First pass of pdflatex
pdflatex -interaction=nonstopmode -output-format pdf ${TEX_FILE}
# Second pass of pdflatex
pdflatex -interaction=nonstopmode -output-format pdf ${TEX_FILE}
#===============================================================================


# The end!
cat <<EOF

The GPUBenchmark results pdf can be found on the following file:
 '${GB_DIR}/${TEX_FILE%.tex}.pdf'"

EOF
